package development

import (
	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/enterprise/src/development/model"
	"shylinux.com/x/enterprise/src/guanlixitong"
)

type Table struct {
	guanlixitong.Table
	list string `name:"list repos_uid uid auto" role:"void"`
}

func (s Table) Inputs(m *ice.Message, arg ...string) {
	switch arg[0] {
	case model.USER_REPOS_ROLE:
		s.InputsListRole(m, UserReposRoleList, arg...)
	case model.REPOS_TYPE:
		s.InputsList(m, ReposTypeList, arg...)
	default:
		s.Table.Inputs(m, arg...)
	}
}
func (s Table) RewriteAppend(m *ice.Message, arg ...string) *ice.Message {
	m.RewriteAppend(func(value, key string, index int) string {
		switch key {
		case model.USER_REPOS_ROLE:
			value = UserReposRole(kit.Int(value)).String()
		case model.REPOS_TYPE:
			value = ReposType(kit.Int(value)).String()
		}
		return value
	})
	return s.Table.RewriteAppend(m)
}
func (s Table) CheckRole(m *ice.Message, arg ...string) *ice.Message {
	role := UserReposRole(kit.Int(m.Cmd(userRepos{}, s.Select, m.OptionSimple(model.REPOS_UID, model.USER_UID)).Append(model.ROLE)))
	m.WarnNotRight(!kit.IsIn(role.String(), append(arg, UserReposCreator.String())...), role.String())
	return m
}
func (s Table) recordEvent(m *ice.Message, info string, arg ...string) {
	s.Table.RecordEvent(m, m.Option(model.REPOS_UID), info, kit.Select(m.Option(model.UID), arg, 0))
}
func (s Table) recordEventWithName(m *ice.Message, info string, arg ...string) {
	s.Table.RecordEventWithName(m, m.Option(model.REPOS_UID), info)
}

type Tables struct{ Table }

func (s Tables) BeforeMigrate(m *ice.Message, arg ...string) {}
