package gongyinglian

import (
	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/mdb"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/enterprise/src/gongyinglian/model"
	"shylinux.com/x/enterprise/src/guanlixitong"
)

type Table struct {
	guanlixitong.Table
	list string `name:"list store_uid uid auto" role:"void"`
}

func (s Table) Inputs(m *ice.Message, arg ...string) {
	switch arg[0] {
	case model.USER_STORE_ROLE:
		s.InputsListRole(m, UserStoreRoleList, arg...)
	case model.STORE_TYPE:
		s.InputsList(m, StoreTypeList, arg...)
	case model.PRODUCT_TYPE:
		s.InputsList(m, ProductTypeList, arg...)
	case model.EXPENSE_TYPE:
		s.InputsList(m, ExpenseTypeList, arg...)
	case model.LOAN_TYPE:
		s.InputsList(m, LoanTypeList, arg...)
	case model.FROM_STORE_UID, model.TO_STORE_UID:
		m.Cmdy(userStore{}, m.Option(model.USER_UID)).Cut(model.STORE_UID, model.STORE_NAME).RenameAppend(model.STORE_UID, arg[0])
		m.DisplayInputKeyNameIconTitle()
	default:
		s.Table.Inputs(m, arg...)
	}
}
func (s Table) RewriteAppend(m *ice.Message, arg ...string) *ice.Message {
	m.RewriteAppend(func(value, key string, index int) string {
		switch key {
		case model.USER_STORE_ROLE:
			value = UserStoreRole(kit.Int(value)).String()
		case model.STORE_TYPE:
			value = StoreType(kit.Int(value)).String()
		case model.PRODUCT_TYPE:
			value = ProductType(kit.Int(value)).String()
		case model.ORDER_TYPE:
			value = OrderType(kit.Int(value)).String()
		case model.ORDER_STATUS:
			value = OrderStatus(kit.Int(value)).String()
		case model.PAYMENT_STATUS:
			value = PaymentStatus(kit.Int(value)).String()
		case model.EXPRESS_STATUS:
			value = ExpressStatus(kit.Int(value)).String()
		case model.RETURN_STATUS:
			value = ReturnStatus(kit.Int(value)).String()
		case model.REFUND_STATUS:
			value = RefundStatus(kit.Int(value)).String()
		case model.EXPENSE_TYPE:
			value = ExpenseType(kit.Int(value)).String()
		case model.LOAN_TYPE:
			value = LoanType(kit.Int(value)).String()
		case model.AMOUNT, model.PRICE:
			value = kit.Format("%0.2f", float64(kit.Int(value))/100)
		}
		return value
	})
	return s.Table.RewriteAppend(m)
}
func (s TableView) transPrice(m *ice.Message, arg ...string) []string {
	for i := 0; i < len(arg); i += 2 {
		switch arg[i] {
		case model.AMOUNT, model.PRICE:
			arg[i+1] = kit.Format(kit.Int(kit.Float(arg[i+1]) * 100))
		}
	}
	return arg
}
func (s Table) CheckRole(m *ice.Message, arg ...string) *ice.Message {
	role := UserStoreRole(kit.Int(m.Cmd(userStore{}, s.Select, m.OptionSimple(model.STORE_UID, model.USER_UID)).Append(model.ROLE)))
	m.WarnNotRight(!kit.IsIn(role.String(), append(arg, UserStoreCreator.String())...), role.String())
	return m
}
func (s Table) checkRole(m *ice.Message, arg ...UserStoreRole) bool {
	kit.If(len(arg) == 0, func() { arg = append(arg, UserStoreBoss) })
	role := s.userStoreRole(m, m.Option(model.STORE_UID))
	for _, v := range append(arg, UserStoreCreator) {
		if role == v {
			return true
		}
	}
	return false
}
func (s Table) userStoreRole(m *ice.Message, arg ...string) UserStoreRole {
	msg := m.Spawn()
	userStore{}.Fields(msg, model.ROLE)
	msg = msg.Cmd(userStore{}, s.Select, model.STORE_UID, arg[0], model.USER_UID, m.Option(model.USER_UID))
	if msg.Length() > 0 {
		return UserStoreRole(kit.Int(msg.Append(model.ROLE)))
	} else {
		return -1
	}
}
func (s Table) checkListRole(m *ice.Message, arg ...string) bool {
	if (len(arg) == 0 && m.Option(model.STORE_UID) != "" && !s.checkRole(m, UserStoreBoss)) || (len(arg) == 1 && !s.checkRole(m.Options(model.STORE_UID, arg[0]), UserStoreBoss)) {
		return false
	}
	return true
}
func (s Table) checkOrderRole(m *ice.Message, arg ...string) bool {
	user_uid := m.Option(model.USER_UID)
	msg := m.Cmd(Order{}, s.Select, "(from_store_uid = ? OR to_store_uid = ?) AND uid = ?", arg[0], arg[0], arg[1])
	if msg.Append(model.FROM_USER_UID) != "" && msg.Append(model.TO_USER_UID) != "" && user_uid != msg.Append(model.FROM_USER_UID) && user_uid != msg.Append(model.TO_USER_UID) {
		return false
	}
	return true
}
func (s Table) recordEvent(m *ice.Message, info string, arg ...string) {
	s.Table.RecordEvent(m, m.Option(model.STORE_UID), info, kit.Select(m.Option(model.UID), arg, 0))
}
func (s Table) recordEventWithName(m *ice.Message, info string) {
	s.Table.RecordEventWithName(m, m.Option(model.STORE_UID), info)
}

type TableView struct {
	Table
	fields string `data:"name,info"`
	create string `name:"create name info" role:"boss"`
	modify string `name:"modify name info" role:"boss"`
	remove string `name:"remove" role:"boss"`
}

func (s TableView) Create(m *ice.Message, arg ...string) {
	arg = s.transPrice(m, arg...)
	s.Table.Create(m, kit.Simple(arg, m.OptionSimple(model.STORE_UID))...)
	s.recordEventWithName(m, "")
}
func (s TableView) Modify(m *ice.Message, arg ...string) {
	arg = s.transPrice(m, arg...)
	s.Table.Update(m, kit.Dict(arg), m.OptionSimple(model.STORE_UID, model.UID)...)
	s.recordEventWithName(m, "")
}
func (s TableView) Remove(m *ice.Message, arg ...string) {
	s.Table.Delete(m, m.OptionSimple(model.STORE_UID, model.UID)...)
	s.recordEventWithName(m, "")
}
func (s TableView) List(m *ice.Message, arg ...string) {
	if len(arg) == 0 {
		s.Select(m, m.OptionSimple(model.STORE_UID)...)
		return
	}
	kit.If(m.Config(mdb.FIELDS), func(p string) { s.FieldsWithCreatedAT(m, "", kit.TransArgs(kit.Split(p, ",", ","))...) })
	if len(arg) == 1 {
		s.Select(m, model.STORE_UID, arg[0])
		kit.If(m.Length() == 0, func() { m.EchoInfoButton("", s.Create) })
	} else if len(arg) == 2 {
		s.SelectDetail(m, model.STORE_UID, arg[0], model.UID, arg[1])
	}
	m.PushAction(s.Modify, s.Remove)
}

type Tables struct{ Table }

func (s Tables) BeforeMigrate(m *ice.Message, arg ...string) {}

func newTable() Table { return Table{Table: guanlixitong.NewTable(userStore{}, store{})} }
