package model

import "shylinux.com/x/mysql-story/src/db"

const (
	UID                  = "uid"
	NAME                 = "name"
	INFO                 = "info"
	TYPE                 = "type"
	ROLE                 = "role"
	STATUS               = "status"
	TITLE                = "title"
	CONTENT              = "content"
	ADDRESS              = "address"
	OPEN_ID              = "open_id"
	USER_UID             = "user_uid"
	USER_STORE_ROLE      = "user_store_role"
	STORE_UID            = "store_uid"
	STORE_NAME           = "store_name"
	STORE_TYPE           = "store_type"
	SHOP_NAME            = "shop_name"
	WAREHOUSE_NAME       = "warehouse_name"
	QUALITY_NAME         = "quality_name"
	BRAND_NAME           = "brand_name"
	PRODUCT_UID          = "product_uid"
	PRODUCT_TYPE         = "product_type"
	ORDER_UID            = "order_uid"
	ORDER_TYPE           = "order_type"
	ORDER_STATUS         = "order_status"
	ORDER_DETAIL_UID     = "order_detail_uid"
	FROM_STORE_UID       = "from_store_uid"
	FROM_USER_UID        = "from_user_uid"
	TO_STORE_UID         = "to_store_uid"
	TO_USER_UID          = "to_user_uid"
	PAYMENT_STATUS       = "payment_status"
	EXPRESS_STATUS       = "express_status"
	RETURN_STATUS        = "return_status"
	REFUND_STATUS        = "refund_status"
	PAYMENT_SUCCESS_TIME = "payment_success_time"
	EXPRESS_RECV_TIME    = "express_recv_time"
	RETURN_RECV_TIME     = "return_recv_time"
	REFUND_SUCCESS_TIME  = "refund_success_time"
	EXPENSE_TYPE         = "expense_type"
	LOAN_TYPE            = "loan_type"
	COMPANY_UID          = "company_uid"
	CITY_UID             = "city_uid"
	AMOUNT               = "amount"
	TOTAL                = "total"
	PRICE                = "price"
	STOCK                = "stock"
	COUNT                = "count"
	UNIT                 = "unit"
	INIT                 = "init"
)

type UserStore struct {
	db.ModelWithUID
	UserUID  string `gorm:"type:char(32);index"`
	StoreUID string `gorm:"type:char(32);index"`
	Role     uint8  `gorm:"default:0"`
	Init     uint8  `gorm:"default:0"`
}
type Store struct {
	db.ModelWithUID
	CompanyUID string `gorm:"type:char(32);index"`
	Name       string `gorm:"type:varchar(64)"`
	Type       uint8  `gorm:"default:0"`
}
type Shop struct {
	db.ModelWithUID
	StoreUID string `gorm:"type:char(32);index"`
	Name     string `gorm:"type:varchar(64)"`
	Address  string `gorm:"type:varchar(255)"`
}
type Warehouse struct {
	db.ModelWithUID
	StoreUID string `gorm:"type:char(32);index"`
	Name     string `gorm:"type:varchar(64)"`
	Address  string `gorm:"type:varchar(255)"`
}
type Product struct {
	db.ModelWithUID
	StoreUID     string `gorm:"type:char(32);index"`
	ShopUID      string `gorm:"type:char(32);index"`
	WarehouseUID string `gorm:"type:char(32);index"`
	QualityUID   string `gorm:"type:char(32);index"`
	BrandUID     string `gorm:"type:char(32);index"`
	Name         string `gorm:"type:varchar(64)"`
	Info         string
	Type         uint8  `gorm:"default:0"`
	Price        uint   `gorm:"default:0"`
	Stock        uint   `gorm:"default:0"`
	Unit         string `gorm:"type:varchar(8)"`
}
type Order struct {
	db.ModelWithUID
	FromStoreUID       string `gorm:"type:char(32);index"`
	FromUserUID        string `gorm:"type:char(32);index"`
	ToStoreUID         string `gorm:"type:char(32);index"`
	ToUserUID          string `gorm:"type:char(32);index"`
	PaymentStatus      uint8  `gorm:"default:0"`
	ExpressStatus      uint8  `gorm:"default:0"`
	ReturnStatus       uint8  `gorm:"default:0"`
	RefundStatus       uint8  `gorm:"default:0"`
	PaymentSuccessTime db.Time
	ExpressRecvTime    db.Time
	ReturnRecvTime     db.Time
	RefundSuccessTime  db.Time
	Name               string `gorm:"type:varchar(64)"`
	Info               string
	Total              int   `gorm:"default:0"`
	Amount             int   `gorm:"default:0"`
	Status             uint8 `gorm:"default:0"`
}
type OrderDetail struct {
	db.ModelWithUID
	StoreUID   string `gorm:"type:char(32);index"`
	OrderUID   string `gorm:"type:char(32);index"`
	ProductUID string `gorm:"type:char(32)"`
	Price      int    `gorm:"default:0"`
	Count      int    `gorm:"default:0"`
}
type Payment struct {
	db.ModelWithUID
	OrderUID   string `gorm:"type:char(32);index"`
	CompanyUID string `gorm:"type:char(32)"`
	OpenID     string `gorm:"type:varchar(128)"`
	Status     uint8  `gorm:"default:0"`
}
type Express struct {
	db.ModelWithUID
	OrderUID   string `gorm:"type:char(32);index"`
	CompanyUID string `gorm:"type:char(32)"`
	OpenID     string `gorm:"type:varchar(128)"`
	Status     uint8  `gorm:"default:0"`
}
type Return struct {
	db.ModelWithUID
	OrderUID   string `gorm:"type:char(32);index"`
	ExpressUID string `gorm:"type:char(32)"`
	CompanyUID string `gorm:"type:char(32)"`
	OpenID     string `gorm:"type:varchar(128)"`
	Status     uint8  `gorm:"default:0"`
}
type Refund struct {
	db.ModelWithUID
	OrderUID   string `gorm:"type:char(32);index"`
	PaymentUID string `gorm:"type:char(32)"`
	CompanyUID string `gorm:"type:char(32)"`
	OpenID     string `gorm:"type:varchar(128)"`
	Status     uint8  `gorm:"default:0"`
}

func init() {
	db.CmdModels("",
		&UserStore{}, &Store{}, &Shop{}, &Warehouse{}, &Product{},
		&Order{}, &OrderDetail{}, &Payment{}, &Express{}, &Return{}, &Refund{},
		&Brand{}, &Quality{}, &Expense{}, &Loan{},
	)
}

type Brand struct {
	db.ModelWithUID
	StoreUID string `gorm:"type:char(32);index"`
	Name     string `gorm:"type:varchar(64)"`
	Info     string
}
type Quality struct {
	db.ModelWithUID
	StoreUID string `gorm:"type:char(32);index"`
	Name     string `gorm:"type:varchar(64)"`
	Info     string
}
type Expense struct {
	db.ModelWithUID
	StoreUID    string `gorm:"type:char(32);index"`
	FromUserUID string `gorm:"type:char(32)"`
	ToUserUID   string `gorm:"type:char(32)"`
	Name        string `gorm:"type:varchar(64)"`
	Info        string
	Type        uint8 `gorm:"default:0"`
	Amount      int   `gorm:"default:0"`
}
type Loan struct {
	db.ModelWithUID
	StoreUID    string `gorm:"type:char(32);index"`
	FromUserUID string `gorm:"type:char(32)"`
	ToUserUID   string `gorm:"type:char(32)"`
	Name        string `gorm:"type:varchar(64)"`
	Info        string
	Type        uint8 `gorm:"default:0"`
	Amount      int   `gorm:"default:0"`
}
